@@ -166,6 +166,9 @@ sub _connect {
 sub _handle_error {
     require Carp;
 
+    # Cut down the error string to a reasonable size
+    $_[0] = substr($_[0], 0, 2000) . ' ... ' . substr($_[0], -2000)
+        if length($_[0]) > 4000;
     $_[0] = Carp::longmess($_[0]);
     return 0; # Now let DBI handle raising the error
 }
