@@ -38,18 +38,15 @@ use Bugzilla::BugMail;
 
 $::lockcount = 0;
 
-my $cgi = Bugzilla->cgi;
-
 unless ( Param("move-enabled") ) {
-  print $cgi->header();
-  PutHeader("Move Bugs");
   print "\n<P>Sorry. Bug moving is not enabled here. ";
   print "If you need to move a bug, contact " . Param("maintainer");
-  PutFooter();
   exit;
 }
 
-my $user = Bugzilla->login(LOGIN_REQUIRED);
+Bugzilla->login(LOGIN_REQUIRED);
+
+my $cgi = Bugzilla->cgi;
 
 sub Log {
     my ($str) = (@_);
@@ -93,7 +90,11 @@ if (!defined $cgi->param('buglist')) {
   exit;
 }
 
-unless ($user->is_mover) {
+my $exporter = Bugzilla->user->login;
+my $movers = Param("movers");
+$movers =~ s/\s?,\s?/|/g;
+$movers =~ s/@/\@/g;
+unless ($exporter =~ /($movers)/) {
   print $cgi->header();
   PutHeader("Move Bugs");
   print "<P>You do not have permission to move bugs<P>\n";
@@ -102,15 +103,14 @@ unless ($user->is_mover) {
 }
 
 my @bugs;
-my $exporterid = $user->id;
-
-print $cgi->header();
-PutHeader("Move Bugs");
 
+print "<P>\n";
 foreach my $id (split(/:/, scalar($cgi->param('buglist')))) {
-  my $bug = new Bugzilla::Bug($id, $exporterid);
+  my $bug = new Bugzilla::Bug($id, $::userid);
   push @bugs, $bug;
   if (!$bug->error) {
+    my $exporterid = DBNameToIdAndCheck($exporter);
+
     my $fieldid = GetFieldID("bug_status");
     my $cur_status= $bug->bug_status;
     SendSQL("INSERT INTO bugs_activity " .
@@ -130,7 +130,7 @@ foreach my $id (split(/:/, scalar($cgi->param('buglist')))) {
         $comment .= $cgi->param('comment') . "\n\n";
     }
     $comment .= "Bug moved to " . Param("move-to-url") . ".\n\n";
-    $comment .= "If the move succeeded, " . $user->login . " will receive a mail\n";
+    $comment .= "If the move succeeded, $exporter will receive a mail\n";
     $comment .= "containing the number of the new bug in the other database.\n";
     $comment .= "If all went well,  please mark this bug verified, and paste\n";
     $comment .= "in a link to the new bug. Otherwise, reopen this bug.\n";
@@ -138,9 +138,10 @@ foreach my $id (split(/:/, scalar($cgi->param('buglist')))) {
         "($id,  $exporterid, now(), " . SqlQuote($comment) . ")");
 
     print "<P>Bug $id moved to " . Param("move-to-url") . ".<BR>\n";
-    Bugzilla::BugMail::Send($id, { 'changer' => $user->login });
+    Bugzilla::BugMail::Send($id, { 'changer' => $exporter });
   }
 }
+print "<P>\n";
 
 my $buglist = $cgi->param('buglist');
 $buglist =~ s/:/,/g;
@@ -167,8 +168,7 @@ $template->process("bug/show.xml.tmpl", { bugs => \@bugs,
 
 $msg .= "\n";
 
-Bugzilla::BugMail::MessageToMTA($msg, $to);
+Bugzilla::BugMail::MessageToMTA($msg);
 
 my $logstr = "XML: bugs $buglist sent to $to";
 Log($logstr);
-PutFooter();
